package com.bwie.auth.service.impl;

import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.client.naming.utils.RandomUtils;
import com.bwie.auth.feign.UserFeign;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @Author：杨鹏
 * @Package：com.bwie.auth.service.impl
 * @Project：yp_zhoukao86
 * @name：AuthServiceImpl
 * @Date：2024/8/6 9:55
 */
@Service
public class AuthServiceImpl implements AuthService {


    @Autowired
    private UserFeign userFeign;

    @Autowired
    private HttpServletRequest request;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Override
    public Result<Integer> getCode(User user) {
        if (StringUtils.isAllEmpty(user.getUserPhone())){
            return Result.error("手机号不能为空");
        }

        Result<User> login = userFeign.login(user);
        if (login == null ){
            return Result.error("手机号不存在");
        }
        int code = RandomUtils.nextInt(9999);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_CODE_KEY+user.getUserPhone(),
                JSONObject.toJSONString(code),
                TokenConstants.REFRESH_TIME,
                TimeUnit.MINUTES);
        return Result.success(code,"验证码发送成功");
    }

    @Override
    public Result<User> login(User user) {
        if (StringUtils.isAllEmpty(user.getUserPhone(),user.getCode())){
            return Result.error("手机号/验证码不能为空");
        }

        Result<User> login = userFeign.login(user);
        User data = login.getData();
        if (data == null ){
            return Result.error("手机号不存在");
        }

        String code = redisTemplate.opsForValue().get(TokenConstants.LOGIN_CODE_KEY + user.getUserPhone());

        if (!user.getCode().equals(code)){
            return Result.error("验证码错");
        }

        String userKey = UUID.randomUUID().toString().replace("-", "");
        Map<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY,userKey);
        String token = JwtUtils.createToken(map);
        data.setToken(token);

        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+userKey,JSONObject.toJSONString(data),
                TokenConstants.EXPIRATION, TimeUnit.MINUTES);
        return Result.success(data);
    }

    @Override
    public Result<User> info() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return Result.success(JSONObject.parseObject(s, User.class));
    }

    @Override
    public void logout() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        redisTemplate.delete(TokenConstants.LOGIN_TOKEN_KEY + userKey);
    }


}
