package com.bwie.firm.consumer;

import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * @Author：杨鹏
 * @Package：com.bwie.firm.consumer
 * @Project：yp_zhoukao86
 * @name：FirmConsumer
 * @Date：2024/8/6 15:34
 */
@Log4j2
@Component
public class FirmConsumerBai {

    @Autowired
    private StringRedisTemplate redisTemplate;

    private static final String INDEX_SET = "INDEX_SET";

    @RabbitListener(queuesToDeclare = {@Queue("INDEX_MQ_BAI")})
    public void firmConsumerBai(String userPhone, Message message, Channel channel){

        //iv.	考虑消息队列中消息重复消费以及消息丢失问题（5分）
        try {
            String messageId = message.getMessageProperties().getMessageId();
            Long add = redisTemplate.opsForSet().add(INDEX_SET, messageId);
            if (add == 1){
                log.info("消费者开始消费,消费内容为{},",userPhone);
                log.info("{}",userPhone);
                log.info("消费者消费成功,消费内容为{},",userPhone);
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
            }
        } catch (Exception e) {
            log.info("报错为:{}",e);
            try {
                channel.basicReject(message.getMessageProperties().getDeliveryTag(),true);
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }


    }

}
