package com.bwie.firm.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Firm;
import com.bwie.common.domain.Job;
import com.bwie.common.domain.People;
import com.bwie.common.domain.User;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.firm.mapper.FirmMapper;
import com.bwie.firm.service.FirmService;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @Author：杨鹏
 * @Package：com.bwie.user.service.impl
 * @Project：yp_zhoukao86
 * @name：FirmServiceImpl
 * @Date：2024/8/6 10:40
 */
@Service
public class FirmServiceImpl implements FirmService {
    private static final String LIST_JOB = "job";

    @Autowired
    private FirmMapper firmMapper;

    @Autowired
    private HttpServletRequest request;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Override
    public Result firmList(Firm firm) {
        List<Firm> firmList = firmMapper.firmList(firm);
        return Result.success(firmList);
    }

    @Override
    public Result peopleList(People people) {
        List<People> peopleList = firmMapper.peopleList(people);
        return Result.success(peopleList);
    }

    @Transactional
    @Override
    public Result peopleAdd(People people) {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSONObject.parseObject(s, User.class);
        people.setUserId(user.getUserId());

        //g)	点击投个简历按钮执行以下逻辑
        People people1 = firmMapper.listPeopleId(people);
        if (people1 == null){
            //i.	在该岗位的候选人列表中插入一条记录（2分）
            firmMapper.peopleAdd(people);
            //ii.	插入成功后，更新企业招聘列表中的候选人数加一（2分）
            firmMapper.updFirmCount(people.getFirmId());
            //iii.	更新个人简历表中的已投递岗位数量加一，最近投递时间列更新为系统当前时间（3分）
            firmMapper.updUserCount(people.getUserId());
            return Result.success();
        }
        //iv.	如果该岗位当前登录用户已投递，则提示用户：该岗位您已投递过，请耐心等待！（5分）
        return Result.error("该岗位您已投递过，请耐心等待！");
    }

    @Override
    public Result peopleUpd(People people) {
        //1.	把该条记录的状态改为已通过（2分）
        //1.	把该条记录的状态改为不通过（2分）
        firmMapper.updPeopleStatus(people);
        User user = firmMapper.listUserId(people.getUserId());
        if (people.getPeopleStatus().equals("1")){
            //2.	通过中间件异步调用消息服务发送短信给候选人，提示：恭喜你，你的简历已通过，请前来面试！（注意：在消费者打印短信内容并且有演示，短信内容自拟，能够成功发送即可，否则0分）（3分）
            rabbitTemplate.convertAndSend("INDEX_MQ_GONG",user.getUserPhone(),message -> {
                message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
                return message;
            });
        }
        //2.	通过中间件异步调用消息服务发送短信给候选人，提示：很遗憾，你的简历不符合岗位要求，请再接再厉！（注意：在消费者打印短信内容并且有演示，短信内容自拟，能够成功发送即可，否则0分）（3分）
        rabbitTemplate.convertAndSend("INDEX_MQ_BAI",user.getUserPhone(),message -> {
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            return message;
        });
        return Result.success();
    }

    @Override
    public Result jobList() {
        String s = redisTemplate.opsForValue().get(LIST_JOB);
        if (s == null){
            return Result.success(JSONObject.parseObject(s, Job.class));
        }
        List<Job> jobList = firmMapper.jobList();
        redisTemplate.opsForValue().set(LIST_JOB,JSONObject.toJSONString(jobList),
                TokenConstants.REFRESH_TIME, TimeUnit.MINUTES);
        return Result.success(jobList);
    }
}
